'use strict';
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
const system_1 = require("./system");
const vscode_1 = require("vscode");
const blameAnnotationProvider_1 = require("./blameAnnotationProvider");
const comparers_1 = require("./comparers");
const constants_1 = require("./constants");
const logger_1 = require("./logger");
class BlameAnnotationController extends vscode_1.Disposable {
    constructor(context, git) {
        super(() => this.dispose());
        this.context = context;
        this.git = git;
        this._annotationProviders = new Map();
        this._pendingClearAnnotations = new Map();
        this._pendingWhitespaceToggles = new Set();
    }
    dispose() {
        for (const fn of this._pendingClearAnnotations.values()) {
            fn.cancel();
        }
        this._annotationProviders.forEach((p, i) => __awaiter(this, void 0, void 0, function* () { return yield this.clear(i); }));
        this._blameAnnotationsDisposable && this._blameAnnotationsDisposable.dispose();
        this._pendingWhitespaceToggleDisposable && this._pendingWhitespaceToggleDisposable.dispose();
    }
    clear(column, toggleRenderWhitespace = true) {
        return __awaiter(this, void 0, void 0, function* () {
            const provider = this._annotationProviders.get(column);
            if (!provider)
                return;
            this._annotationProviders.delete(column);
            yield provider.dispose(toggleRenderWhitespace);
            if (this._annotationProviders.size === 0) {
                logger_1.Logger.log(`Remove listener registrations for blame annotations`);
                this._blameAnnotationsDisposable && this._blameAnnotationsDisposable.dispose();
                this._blameAnnotationsDisposable = undefined;
            }
        });
    }
    showBlameAnnotation(editor, shaOrLine) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!editor || !editor.document)
                return false;
            if (editor.viewColumn === undefined && !this.git.hasGitUriForFile(editor))
                return false;
            const currentProvider = this._annotationProviders.get(editor.viewColumn || -1);
            if (currentProvider && comparers_1.TextEditorComparer.equals(currentProvider.editor, editor)) {
                yield currentProvider.setSelection(shaOrLine);
                return true;
            }
            const provider = new blameAnnotationProvider_1.BlameAnnotationProvider(this.context, this.git, editor);
            if (!(yield provider.supportsBlame()))
                return false;
            if (currentProvider) {
                yield this.clear(currentProvider.editor.viewColumn || -1, false);
            }
            if (!this._blameAnnotationsDisposable && this._annotationProviders.size === 0) {
                logger_1.Logger.log(`Add listener registrations for blame annotations`);
                const subscriptions = [];
                subscriptions.push(vscode_1.window.onDidChangeVisibleTextEditors(system_1.Functions.debounce(this._onVisibleTextEditorsChanged, 100), this));
                subscriptions.push(vscode_1.window.onDidChangeTextEditorViewColumn(this._onTextEditorViewColumnChanged, this));
                subscriptions.push(vscode_1.workspace.onDidCloseTextDocument(this._onTextDocumentClosed, this));
                this._blameAnnotationsDisposable = vscode_1.Disposable.from(...subscriptions);
                this._visibleColumns = this._getVisibleColumns(vscode_1.window.visibleTextEditors);
            }
            this._annotationProviders.set(editor.viewColumn || -1, provider);
            return provider.provideBlameAnnotation(shaOrLine);
        });
    }
    toggleBlameAnnotation(editor, shaOrLine) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!editor || !editor.document)
                return false;
            if (editor.viewColumn === undefined && !this.git.hasGitUriForFile(editor))
                return false;
            let provider = this._annotationProviders.get(editor.viewColumn || -1);
            if (!provider)
                return this.showBlameAnnotation(editor, shaOrLine);
            yield this.clear(provider.editor.viewColumn || -1);
            return false;
        });
    }
    _getVisibleColumns(editors) {
        const set = new Set();
        for (const e of editors) {
            if (e.viewColumn === undefined && !this.git.hasGitUriForFile(e))
                continue;
            set.add(e.viewColumn);
        }
        return set;
    }
    _onActiveTextEditorChanged(e) {
        if (this._pendingWhitespaceToggles.size === 0 || (e.viewColumn === undefined && !this.git.hasGitUriForFile(e)))
            return;
        const viewColumn = e.viewColumn || -1;
        if (this._pendingWhitespaceToggles.has(viewColumn)) {
            logger_1.Logger.log('ActiveTextEditorChanged:', `Remove pending whitespace toggle for column ${viewColumn}`);
            this._pendingWhitespaceToggles.delete(viewColumn);
            logger_1.Logger.log('ActiveTextEditorChanged:', `Toggle whitespace rendering on`);
            vscode_1.commands.executeCommand(constants_1.BuiltInCommands.ToggleRenderWhitespace);
        }
        if (this._pendingWhitespaceToggles.size === 0) {
            logger_1.Logger.log('ActiveTextEditorChanged:', `Remove listener registrations for pending whitespace toggles`);
            this._pendingWhitespaceToggleDisposable.dispose();
            this._pendingWhitespaceToggleDisposable = undefined;
        }
    }
    _onTextDocumentClosed(e) {
        for (const [key, p] of this._annotationProviders) {
            if (!comparers_1.TextDocumentComparer.equals(p.document, e))
                continue;
            logger_1.Logger.log('TextDocumentClosed:', `Add pending clear of blame annotations for column ${key}`);
            const fn = system_1.Functions.debounce(() => {
                this._pendingClearAnnotations.delete(key);
                this.clear(key);
            }, 250);
            this._pendingClearAnnotations.set(key, fn);
            fn();
        }
    }
    _onTextEditorViewColumnChanged(e) {
        return __awaiter(this, void 0, void 0, function* () {
            this._visibleColumns = this._getVisibleColumns(vscode_1.window.visibleTextEditors);
            const viewColumn = e.viewColumn || -1;
            logger_1.Logger.log('TextEditorViewColumnChanged:', `Clear blame annotations for column ${viewColumn}`);
            yield this.clear(viewColumn);
            for (const [key, p] of this._annotationProviders) {
                if (!comparers_1.TextEditorComparer.equals(p.editor, e.textEditor))
                    continue;
                logger_1.Logger.log('TextEditorViewColumnChanged:', `Clear blame annotations for column ${key}`);
                yield this.clear(key, false);
            }
        });
    }
    _onVisibleTextEditorsChanged(e) {
        return __awaiter(this, void 0, void 0, function* () {
            if (e.every(_ => _.document.uri.scheme === 'inmemory'))
                return;
            this._visibleColumns = this._getVisibleColumns(e);
            for (const [key, fn] of this._pendingClearAnnotations) {
                logger_1.Logger.log('VisibleTextEditorsChanged:', `Remove pending blame annotations for column ${key}`);
                fn.cancel();
                this._pendingClearAnnotations.delete(key);
                logger_1.Logger.log('VisibleTextEditorsChanged:', `Clear blame annotations for column ${key}`);
                yield this.clear(key, this._visibleColumns.has(key));
            }
            for (const [key, p] of this._annotationProviders) {
                if (e.some(_ => comparers_1.TextEditorComparer.equals(p.editor, _)))
                    continue;
                logger_1.Logger.log('VisibleTextEditorsChanged:', `Clear blame annotations for column ${key}`);
                const editor = vscode_1.window.activeTextEditor;
                if (p.requiresRenderWhitespaceToggle && (editor && (editor.viewColumn || -1) !== key)) {
                    this.clear(key, false);
                    if (!this._pendingWhitespaceToggleDisposable) {
                        logger_1.Logger.log('VisibleTextEditorsChanged:', `Add listener registrations for pending whitespace toggles`);
                        this._pendingWhitespaceToggleDisposable = vscode_1.window.onDidChangeActiveTextEditor(this._onActiveTextEditorChanged, this);
                    }
                    logger_1.Logger.log('VisibleTextEditorsChanged:', `Add pending whitespace toggle for column ${key}`);
                    this._pendingWhitespaceToggles.add(key);
                }
                else {
                    this.clear(key);
                }
            }
        });
    }
}
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = BlameAnnotationController;
//# sourceMappingURL=blameAnnotationController.js.map