'use strict';
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
const system_1 = require("./system");
const vscode_1 = require("vscode");
const comparers_1 = require("./comparers");
const configuration_1 = require("./configuration");
const constants_1 = require("./constants");
const gitProvider_1 = require("./gitProvider");
const logger_1 = require("./logger");
const moment = require("moment");
const blameDecoration = vscode_1.window.createTextEditorDecorationType({
    before: {
        margin: '0 1.75em 0 0'
    }
});
let highlightDecoration;
class BlameAnnotationProvider extends vscode_1.Disposable {
    constructor(context, git, editor) {
        super(() => this.dispose());
        this.git = git;
        this.editor = editor;
        this.requiresRenderWhitespaceToggle = false;
        if (!highlightDecoration) {
            highlightDecoration = vscode_1.window.createTextEditorDecorationType({
                dark: {
                    backgroundColor: 'rgba(255, 255, 255, 0.15)',
                    gutterIconPath: context.asAbsolutePath('images/blame-dark.png'),
                    overviewRulerColor: 'rgba(255, 255, 255, 0.75)'
                },
                light: {
                    backgroundColor: 'rgba(0, 0, 0, 0.15)',
                    gutterIconPath: context.asAbsolutePath('images/blame-light.png'),
                    overviewRulerColor: 'rgba(0, 0, 0, 0.75)'
                },
                gutterIconSize: 'contain',
                overviewRulerLane: vscode_1.OverviewRulerLane.Right,
                isWholeLine: true
            });
        }
        this.document = this.editor.document;
        this._uri = gitProvider_1.GitUri.fromUri(this.document.uri, this.git);
        this._blame = this.git.getBlameForFile(this._uri.fsPath, this._uri.sha, this._uri.repoPath);
        this._config = vscode_1.workspace.getConfiguration('gitlens').get('blame');
        const subscriptions = [];
        subscriptions.push(vscode_1.workspace.onDidChangeConfiguration(this._onConfigurationChanged, this));
        subscriptions.push(vscode_1.window.onDidChangeTextEditorSelection(this._onActiveSelectionChanged, this));
        this._disposable = vscode_1.Disposable.from(...subscriptions);
        this._onConfigurationChanged();
    }
    dispose(toggleRenderWhitespace = true) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.editor) {
                this.editor.setDecorations(blameDecoration, []);
                this.editor.setDecorations(highlightDecoration, []);
            }
            if (toggleRenderWhitespace && this.requiresRenderWhitespaceToggle) {
                logger_1.Logger.log('BlameAnnotationProvider.dispose:', `Toggle whitespace rendering on`);
                yield vscode_1.commands.executeCommand(constants_1.BuiltInCommands.ToggleRenderWhitespace);
            }
            this._disposable && this._disposable.dispose();
        });
    }
    _onConfigurationChanged() {
        const renderWhitespace = vscode_1.workspace.getConfiguration('editor').get('renderWhitespace');
        this.requiresRenderWhitespaceToggle = !(renderWhitespace == null || renderWhitespace === 'none');
    }
    _onActiveSelectionChanged(e) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!comparers_1.TextDocumentComparer.equals(this.document, e.textEditor && e.textEditor.document))
                return;
            return this.setSelection(e.selections[0].active.line);
        });
    }
    supportsBlame() {
        return __awaiter(this, void 0, void 0, function* () {
            const blame = yield this._blame;
            return !!(blame && blame.lines.length);
        });
    }
    provideBlameAnnotation(shaOrLine) {
        return __awaiter(this, void 0, void 0, function* () {
            const blame = yield this._blame;
            if (!blame || !blame.lines.length)
                return false;
            if (this.requiresRenderWhitespaceToggle) {
                logger_1.Logger.log('BlameAnnotationProvider.provideBlameAnnotation:', `Toggle whitespace rendering off`);
                yield vscode_1.commands.executeCommand(constants_1.BuiltInCommands.ToggleRenderWhitespace);
            }
            let blameDecorationOptions;
            switch (this._config.annotation.style) {
                case configuration_1.BlameAnnotationStyle.Compact:
                    blameDecorationOptions = this._getCompactGutterDecorations(blame);
                    break;
                case configuration_1.BlameAnnotationStyle.Expanded:
                    blameDecorationOptions = this._getExpandedGutterDecorations(blame);
                    break;
            }
            if (blameDecorationOptions) {
                this.editor.setDecorations(blameDecoration, blameDecorationOptions);
            }
            this._setSelection(blame, shaOrLine);
            return true;
        });
    }
    setSelection(shaOrLine) {
        return __awaiter(this, void 0, void 0, function* () {
            const blame = yield this._blame;
            if (!blame || !blame.lines.length)
                return;
            return this._setSelection(blame, shaOrLine);
        });
    }
    _setSelection(blame, shaOrLine) {
        const offset = this._uri.offset;
        let sha;
        if (typeof shaOrLine === 'string') {
            sha = shaOrLine;
        }
        else if (typeof shaOrLine === 'number') {
            const line = shaOrLine - offset;
            if (line >= 0) {
                sha = blame.lines[line].sha;
            }
        }
        else {
            sha = system_1.Iterables.first(blame.commits.values()).sha;
        }
        if (!sha) {
            this.editor.setDecorations(highlightDecoration, []);
            return;
        }
        const highlightDecorationRanges = blame.lines
            .filter(l => l.sha === sha)
            .map(l => this.editor.document.validateRange(new vscode_1.Range(l.line + offset, 0, l.line + offset, 1000000)));
        this.editor.setDecorations(highlightDecoration, highlightDecorationRanges);
    }
    _getCompactGutterDecorations(blame) {
        const offset = this._uri.offset;
        let count = 0;
        let lastSha;
        return blame.lines.map(l => {
            let color = l.previousSha ? '#999999' : '#6b6b6b';
            let commit = blame.commits.get(l.sha);
            let hoverMessage = [`_${l.sha}_ - ${commit.message}`, `${commit.author}, ${moment(commit.date).format('MMMM Do, YYYY h:MMa')}`];
            if (commit.isUncommitted) {
                color = 'rgba(0, 188, 242, 0.6)';
                let previous = blame.commits.get(commit.previousSha);
                if (previous) {
                    hoverMessage = ['Uncommitted changes', `_${previous.sha}_ - ${previous.message}`, `${previous.author}, ${moment(previous.date).format('MMMM Do, YYYY h:MMa')}`];
                }
                else {
                    hoverMessage = ['Uncommitted changes', `_${l.previousSha}_`];
                }
            }
            let gutter = '';
            if (lastSha !== l.sha) {
                count = -1;
            }
            const isEmptyOrWhitespace = this.document.lineAt(l.line).isEmptyOrWhitespace;
            if (!isEmptyOrWhitespace) {
                switch (++count) {
                    case 0:
                        gutter = commit.sha.substring(0, 8);
                        break;
                    case 1:
                        gutter = `\\00a6\\00a0 ${this._getAuthor(commit, 17, true)}`;
                        break;
                    case 2:
                        gutter = `\\00a6\\00a0 ${this._getDate(commit, true)}`;
                        break;
                    default:
                        gutter = '\\00a6\\00a0';
                        break;
                }
            }
            lastSha = l.sha;
            return {
                range: this.editor.document.validateRange(new vscode_1.Range(l.line + offset, 0, l.line + offset, 0)),
                hoverMessage: hoverMessage,
                renderOptions: { before: { color: color, contentText: gutter, width: '11em' } }
            };
        });
    }
    _getExpandedGutterDecorations(blame) {
        const offset = this._uri.offset;
        let width = 0;
        if (this._config.annotation.sha) {
            width += 5;
        }
        if (this._config.annotation.date) {
            if (width > 0) {
                width += 7;
            }
            else {
                width += 6;
            }
        }
        if (this._config.annotation.author) {
            if (width > 5 + 6) {
                width += 12;
            }
            else if (width > 0) {
                width += 11;
            }
            else {
                width += 10;
            }
        }
        return blame.lines.map(l => {
            let color = l.previousSha ? '#999999' : '#6b6b6b';
            let commit = blame.commits.get(l.sha);
            let hoverMessage = [`_${l.sha}_ - ${commit.message}`, `${commit.author}, ${moment(commit.date).format('MMMM Do, YYYY h:MMa')}`];
            if (commit.isUncommitted) {
                color = 'rgba(0, 188, 242, 0.6)';
                let previous = blame.commits.get(commit.previousSha);
                if (previous) {
                    hoverMessage = ['Uncommitted changes', `_${previous.sha}_ - ${previous.message}`, `${previous.author}, ${moment(previous.date).format('MMMM Do, YYYY h:MMa')}`];
                }
                else {
                    hoverMessage = ['Uncommitted changes', `_${l.previousSha}_`];
                }
            }
            const gutter = this._getGutter(commit);
            return {
                range: this.editor.document.validateRange(new vscode_1.Range(l.line + offset, 0, l.line + offset, 0)),
                hoverMessage: hoverMessage,
                renderOptions: { before: { color: color, contentText: gutter, width: `${width}em` } }
            };
        });
    }
    _getAuthor(commit, max = 17, force = false) {
        if (!force && !this._config.annotation.author)
            return '';
        let author = commit.isUncommitted ? 'Uncommitted' : commit.author;
        if (author.length > max) {
            return `${author.substring(0, max - 1)}\\2026`;
        }
        return author;
    }
    _getDate(commit, force) {
        if (!force && !this._config.annotation.date)
            return '';
        return moment(commit.date).format('MM/DD/YYYY');
    }
    _getGutter(commit) {
        const author = this._getAuthor(commit);
        const date = this._getDate(commit);
        if (this._config.annotation.sha) {
            return `${commit.sha.substring(0, 8)}${(date ? `\\00a0\\2022\\00a0 ${date}` : '')}${(author ? `\\00a0\\2022\\00a0 ${author}` : '')}`;
        }
        else if (this._config.annotation.date) {
            return `${date}${(author ? `\\00a0\\2022\\00a0 ${author}` : '')}`;
        }
        else {
            return author;
        }
    }
}
exports.BlameAnnotationProvider = BlameAnnotationProvider;
//# sourceMappingURL=blameAnnotationProvider.js.map