'use strict';
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
const system_1 = require("./system");
const vscode_1 = require("vscode");
const comparers_1 = require("./comparers");
const configuration_1 = require("./configuration");
const gitProvider_1 = require("./gitProvider");
const logger_1 = require("./logger");
const moment = require("moment");
class BlameStatusBarController extends vscode_1.Disposable {
    constructor(context, git) {
        super(() => this.dispose());
        this.git = git;
        this._onConfigure();
        const subscriptions = [];
        subscriptions.push(vscode_1.workspace.onDidChangeConfiguration(this._onConfigure, this));
        this._disposable = vscode_1.Disposable.from(...subscriptions);
    }
    dispose() {
        this._statusBarDisposable && this._statusBarDisposable.dispose();
        this._statusBarItem && this._statusBarItem.dispose();
        this._disposable && this._disposable.dispose();
    }
    _onConfigure() {
        const config = vscode_1.workspace.getConfiguration('').get('gitlens');
        if (!system_1.Objects.areEquivalent(config.statusBar, this._config && this._config.statusBar)) {
            this._statusBarDisposable && this._statusBarDisposable.dispose();
            this._statusBarItem && this._statusBarItem.dispose();
            if (config.statusBar.enabled) {
                this._statusBarItem = vscode_1.window.createStatusBarItem(vscode_1.StatusBarAlignment.Right, 1000);
                switch (config.statusBar.command) {
                    case configuration_1.StatusBarCommand.ToggleCodeLens:
                        if (config.codeLens.visibility !== 'ondemand') {
                            config.statusBar.command = configuration_1.StatusBarCommand.BlameAnnotate;
                        }
                        break;
                }
                this._statusBarItem.command = config.statusBar.command;
                const subscriptions = [];
                subscriptions.push(vscode_1.window.onDidChangeActiveTextEditor(this._onActiveTextEditorChanged, this));
                subscriptions.push(vscode_1.window.onDidChangeTextEditorSelection(this._onActiveSelectionChanged, this));
                this._statusBarDisposable = vscode_1.Disposable.from(...subscriptions);
            }
            else {
                this._statusBarDisposable = undefined;
                this._statusBarItem = undefined;
            }
        }
        this._config = config;
        this._onActiveTextEditorChanged(vscode_1.window.activeTextEditor);
    }
    _onActiveTextEditorChanged(e) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!e || !e.document || e.document.isUntitled || (e.viewColumn === undefined && !this.git.hasGitUriForFile(e))) {
                this.clear();
                return;
            }
            this._document = e.document;
            this._uri = gitProvider_1.GitUri.fromUri(this._document.uri, this.git);
            const maxLines = this._config.advanced.caching.statusBar.maxLines;
            this._useCaching = this._config.advanced.caching.enabled && (maxLines <= 0 || this._document.lineCount <= maxLines);
            if (this._useCaching) {
                this._blame = this.git.getBlameForFile(this._uri.fsPath, this._uri.sha, this._uri.repoPath);
            }
            else {
                this._blame = undefined;
            }
            return this._showBlame(e.selection.active.line);
        });
    }
    _onActiveSelectionChanged(e) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!comparers_1.TextDocumentComparer.equals(this._document, e.textEditor && e.textEditor.document))
                return;
            return this._showBlame(e.selections[0].active.line);
        });
    }
    _showBlame(line) {
        return __awaiter(this, void 0, void 0, function* () {
            line = line - this._uri.offset;
            let commit;
            if (line >= 0) {
                if (this._useCaching) {
                    const blame = yield this._blame;
                    if (!blame || !blame.lines.length) {
                        this.clear();
                        return;
                    }
                    try {
                        const sha = blame.lines[line].sha;
                        commit = blame.commits.get(sha);
                    }
                    catch (ex) {
                        logger_1.Logger.error(`DEBUG(${this._uri.toString()}): Line ${line} not found in blame; lines=${blame.lines.length}, uriOffset=${this._uri.offset}, repoPath=${blame.repoPath}`);
                    }
                }
                else {
                    const blameLine = yield this.git.getBlameForLine(this._uri.fsPath, line, this._uri.sha, this._uri.repoPath);
                    commit = blameLine && blameLine.commit;
                }
            }
            if (commit) {
                this.show(commit);
            }
            else {
                this.clear();
            }
        });
    }
    clear() {
        this._statusBarItem && this._statusBarItem.hide();
        this._document = undefined;
        this._blame = undefined;
    }
    show(commit) {
        this._statusBarItem.text = `$(git-commit) ${commit.author}, ${moment(commit.date).fromNow()}`;
        switch (this._config.statusBar.command) {
            case configuration_1.StatusBarCommand.BlameAnnotate:
                this._statusBarItem.tooltip = 'Toggle Blame Annotations';
                break;
            case configuration_1.StatusBarCommand.ShowBlameHistory:
                this._statusBarItem.tooltip = 'Open Blame History';
                break;
            case configuration_1.StatusBarCommand.ShowFileHistory:
                this._statusBarItem.tooltip = 'Open File History';
                break;
            case configuration_1.StatusBarCommand.DiffWithPrevious:
                this._statusBarItem.tooltip = 'Compare to Previous Commit';
                break;
            case configuration_1.StatusBarCommand.ToggleCodeLens:
                this._statusBarItem.tooltip = 'Toggle Blame CodeLens';
                break;
            case configuration_1.StatusBarCommand.ShowQuickFileHistory:
                this._statusBarItem.tooltip = 'View Git File History';
                break;
        }
        this._statusBarItem.show();
    }
}
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = BlameStatusBarController;
//# sourceMappingURL=blameStatusBarController.js.map